/* $Id: compat.h,v 1.13 2005/04/09 19:14:31 larsv Exp $
 *
 * ETRAX FS IOP USB 1.1 Host Controller
 *
 * Linux kernel compatibility layer
 *
 * Copyright (C) 2004, 2005  Axis Communications AB
 */

#ifndef __COMPAT_H
#define __COMPAT_H

#ifdef __KERNEL__

#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/init.h>
#include <linux/slab.h>
#include <linux/interrupt.h>
#include <asm/string.h>
#include <asm/io.h>
#include <asm/arch/dma.h>
#include <asm/arch/cache.h>
#include <asm/arch/intmem.h>
#include <asm/arch/pinmux.h>
#include <asm/arch/hwregs/reg_map.h>

#define BUSY_WAIT()		do { if (!in_atomic()) schedule(); } while (0)

#define PHYS_TO_VIRT(pa)	((void *) phys_to_virt((unsigned) pa))
#define VIRT_TO_PHYS(va)	((void *) virt_to_phys(va))

#define STATIC_DMA_DESCR(name, size, align) \
	static char *name##_base = NULL;				      \
	char *name = NULL;						      \
	if (!name##_base)						      \
		name##_base = kmalloc(size + align, GFP_KERNEL);	      \
	if (name##_base)						      \
		name = (char *) ALIGN((unsigned) name##_base, align);		

#define DUMMY_BUF(name, size) \
	static char *name##_virt = NULL;				      \
	char *name = NULL;						      \
	if (!name##_virt)						      \
		name##_virt = crisv32_intmem_alloc(size, L1_CACHE_BYTES);     \
	if (name##_virt)						      \
		name = (char *) crisv32_intmem_virt_to_phys(name##_virt);   
				
#else /* __KERNEL__ */

#include <string.h>
#include <assert.h>
#include "reg_map.h"
#include "simdef.h"

typedef unsigned char		u8;
typedef unsigned short		u16;
typedef unsigned int		u32;

#define barrier()		__asm__ __volatile__("": : :"memory")
#define mb()			barrier()
#define rmb()			mb()
#define wmb()			mb()

#define __init
#define __init_or_module
#define __exit

#ifndef L1_CACHE_BYTES
#define L1_CACHE_BYTES		32
#endif /* L1_CACHE_BYTES */

#define BUG()			FAIL()
#define BUG_ON(c)		ASSERT(!(c))

#define BUSY_WAIT()		do { CPU_SLEEP(100); } while (0)

#define PHYS_TO_VIRT(pa)	((void *) phys2virt((unsigned) pa))
#define VIRT_TO_PHYS(va)	((void *) virt2phys((unsigned) va))

#define PORT_B 0 
#define PORT_C 1
#define PORT_D 2
#define PORT_E 3

#define STATIC_DMA_DESCR(name, size, align) \
       static unsigned char name[size] __attribute__((aligned(align)))

#define DUMMY_BUF(name, size)	static unsigned char name[size]

#endif /* __KERNEL__ */

int iop_request_dma(int dmc, int dma_inst, const char *device_id);
int iop_free_dma(int dma_inst);
int iop_pinmux_alloc(int port, int first_pin, int last_pin);
int iop_pinmux_dealloc(int port, int first_pin, int last_pin);
void iop_usb_phy(int);
void iop_start_clock(void);

#endif /* __COMPAT_H */
